/*
  Device driver for Linux TAP device

  T. Nakagawa
  2010-04-07
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <linux/if.h>
#include <linux/if_tun.h>
#include "device.h"


#define TUNDEV "/dev/net/tun"


static int fd = -1;


/*
  Initialize device
*/
void dev_init(void) {
  struct ifreq ifr;
  char buf[256];

  if ((fd = open(TUNDEV, O_RDWR)) < 0) {
    perror("ERROR: dev_init() failed");
    exit(1);
  }

  memset(&ifr, 0, sizeof(ifr));
  ifr.ifr_flags = (IFF_TAP | IFF_NO_PI);

  if (ioctl(fd, TUNSETIFF, (void *)&ifr) < 0) {
    perror("ERROR: dev_init() failed");
    exit(1);
  }

  sprintf(buf, "ifconfig tap0 inet 192.168.1.1");
  system(buf);

  return;
}


/*
  Receive a packet

  This function timeouts after 100 msec
*/
uint16_t dev_recv(uint8_t *data, uint16_t size) {
  fd_set fds;
  struct timeval tv;

  FD_ZERO(&fds);
  FD_SET(fd, &fds);

  tv.tv_sec = 0;
  tv.tv_usec = 100000;
  if (select(fd + 1, &fds, NULL, NULL, &tv) != 0) {
    return (uint16_t)read(fd, data, size);
  }

  return 0;
}


/*
  Send a packet
*/
void dev_send(uint8_t *data, uint16_t size) {
  write(fd, data, size);

  return;
}
