/*
  dns - usock sample program

  T. Nakagawa
  2010-04-11
*/


#include <stdio.h>
#include <string.h>
#include "usock.h"


struct Dns {
  uint16_t id;
  uint8_t opcode;
  uint8_t rcode;
  uint16_t qdcount;
  uint16_t ancount;
  uint16_t nscount;
  uint16_t adcount;
  uint8_t data[1];
};


static uint8_t addr[] = {192, 168, 0, 1};	/* IP address of DNS server */
#define DOMAIN "www.w3.org"


int main(void) {
  int8_t i, j;
  int8_t sd;
  struct Dns *dns;

  if ((sd = usock_open(USOCK_UDP, 0, 53, addr)) < 0) return 1;

  usock_alloc(sd);
  dns = (struct Dns *)Usock_data;
  dns->id = 0;
  dns->opcode = 0x01;
  dns->rcode = 0x00;
  ((uint8_t *)&dns->qdcount)[0] = 0x00;
  ((uint8_t *)&dns->qdcount)[1] = 0x01;
  dns->ancount = 0;
  dns->nscount = 0;
  dns->adcount = 0;
  for (i = 0, j = -1; i < strlen(DOMAIN) + 1; i++) {
    if (i == strlen(DOMAIN) || DOMAIN[i] == '.') {
      dns->data[1 + j] = (i - j) - 1;
      j = i;
    } else {
      dns->data[1 + i] = DOMAIN[i];
    }
  }
  dns->data[i++] = 0x00;
  dns->data[i++] = 0x00;
  dns->data[i++] = 0x01;
  dns->data[i++] = 0x00;
  dns->data[i++] = 0x01;
  Usock_size = 12 + i;
  usock_send(sd);

  if (usock_recv(sd)) return 1;
  dns = (struct Dns *)Usock_data;
  if (dns->id != 0 || (dns->opcode & 0xfa) != 0x80 || (dns->rcode & 0x0f) != 0x00) return 1;
  if (dns->ancount == 0) return 1;
  i = 0;
  for (j = 0; j < 2; j++) {
    for (; ; ) {
      if (dns->data[i] == 0x00) {
	break;
      } else if (dns->data[i] < 0xc0) {
	i += 1 + dns->data[i];
      } else {
	i += 1;
	break;
      }
    }
    i += 5;
  }
  i += 6;

  printf("%s <-> %d.%d.%d.%d\n", DOMAIN, dns->data[i], dns->data[i + 1], dns->data[i + 2], dns->data[i + 3]);

  usock_close(sd);

  return 0;
}
